@@ -10,9 +10,4 @@ echo "udunits2_path = $PREFIX/lib/libudunits2${SHLIB_EXT}" >> $SITECFG
 
 rm -rf lib/iris/tests/results lib/iris/tests/*.npz
 
-python setup.py install --single-version-externally-managed --record record.txt
-
-
-# Without this line, the OS X build fails reproducibly with
-# https://github.com/conda-forge/iris-feedstock/pull/2#issuecomment-217468453.
-python -c "import iris.fileformats.netcdf; iris.fileformats.netcdf._pyke_kb_engine()"
+$PYTHON -m pip install --no-deps --ignore-installed .
