@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2016, Met Office
+# (C) British Crown Copyright 2010 - 2017, Met Office
 #
 # This file is part of Iris.
 #
@@ -36,6 +36,7 @@ import iris.analysis
 import iris.coords
 import iris.coord_systems
 import iris.exceptions
+from iris.util import _meshgrid
 
 
 # This value is used as a fall-back if the cube does not define the earth
@@ -161,18 +162,18 @@ def rotate_pole(lons, lats, pole_lon, pole_lat):
     return rotated_lon, rotated_lat
 
 
-def _get_lat_lon_coords(cube):
+def _get_lon_lat_coords(cube):
     lat_coords = [coord for coord in cube.coords()
                   if "latitude" in coord.name()]
     lon_coords = [coord for coord in cube.coords()
                   if "longitude" in coord.name()]
     if len(lat_coords) > 1 or len(lon_coords) > 1:
         raise ValueError(
-            "Calling _get_lat_lon_coords() with multiple lat or lon coords"
+            "Calling `_get_lon_lat_coords` with multiple lat or lon coords"
             " is currently disallowed")
     lat_coord = lat_coords[0]
     lon_coord = lon_coords[0]
-    return (lat_coord, lon_coord)
+    return (lon_coord, lat_coord)
 
 
 def _xy_range(cube, mode=None):
@@ -259,7 +260,7 @@ def get_xy_grids(cube):
 
     if x.ndim == y.ndim == 1:
         # Convert to 2D.
-        x, y = np.meshgrid(x, y)
+        x, y = _meshgrid(x, y)
     elif x.ndim == y.ndim == 2:
         # They are already in the correct shape.
         pass
@@ -284,45 +285,45 @@ def get_xy_contiguous_bounded_grids(cube):
 
     x = x_coord.contiguous_bounds()
     y = y_coord.contiguous_bounds()
-    x, y = np.meshgrid(x, y)
+    x, y = _meshgrid(x, y)
 
     return (x, y)
 
 
-def _quadrant_area(radian_colat_bounds, radian_lon_bounds, radius_of_earth):
+def _quadrant_area(radian_lat_bounds, radian_lon_bounds, radius_of_earth):
     """Calculate spherical segment areas.
 
-    - radian_colat_bounds    -- [n,2] array of colatitude bounds (radians)
-    - radian_lon_bounds      -- [n,2] array of longitude bounds (radians)
-    - radius_of_earth        -- radius of the earth
-                                (currently assumed spherical)
+    - radian_lat_bounds    -- [n,2] array of latitude bounds (radians)
+    - radian_lon_bounds    -- [n,2] array of longitude bounds (radians)
+    - radius_of_earth      -- radius of the earth
+                              (currently assumed spherical)
 
     Area weights are calculated for each lat/lon cell as:
 
         .. math::
 
-            r^2 (lon_1 - lon_0) ( cos(colat_0) - cos(colat_1))
+            r^2 (lon_1 - lon_0) ( sin(lat_1) - sin(lat_0))
 
     The resulting array will have a shape of
-    *(radian_colat_bounds.shape[0], radian_lon_bounds.shape[0])*
+    *(radian_lat_bounds.shape[0], radian_lon_bounds.shape[0])*
 
     The calculations are done at 64 bit precision and the returned array
     will be of type numpy.float64.
 
     """
     # ensure pairs of bounds
-    if (radian_colat_bounds.shape[-1] != 2 or
+    if (radian_lat_bounds.shape[-1] != 2 or
             radian_lon_bounds.shape[-1] != 2 or
-            radian_colat_bounds.ndim != 2 or
+            radian_lat_bounds.ndim != 2 or
             radian_lon_bounds.ndim != 2):
         raise ValueError("Bounds must be [n,2] array")
 
     # fill in a new array of areas
     radius_sqr = radius_of_earth ** 2
-    radian_colat_64 = radian_colat_bounds.astype(np.float64)
+    radian_lat_64 = radian_lat_bounds.astype(np.float64)
     radian_lon_64 = radian_lon_bounds.astype(np.float64)
 
-    ylen = np.cos(radian_colat_64[:, 0]) - np.cos(radian_colat_64[:, 1])
+    ylen = np.sin(radian_lat_64[:, 1]) - np.sin(radian_lat_64[:, 0])
     xlen = radian_lon_64[:, 1] - radian_lon_64[:, 0]
     areas = radius_sqr * np.outer(ylen, xlen)
 
@@ -354,7 +355,7 @@ def area_weights(cube, normalize=False):
 
         .. math::
 
-            r^2 cos(lat_0) (lon_1 - lon_0) - r^2 cos(lat_1) (lon_1 - lon_0)
+            r^2 (lon_1 - lon_0) (\sin(lat_1) - \sin(lat_0))
 
     Currently, only supports a spherical datum.
     Uses earth radius from the cube, if present and spherical.
@@ -378,7 +379,7 @@ def area_weights(cube, normalize=False):
 
     # Get the lon and lat coords and axes
     try:
-        lat, lon = _get_lat_lon_coords(cube)
+        lon, lat = _get_lon_lat_coords(cube)
     except IndexError:
         raise ValueError('Cannot get latitude/longitude '
                          'coordinates from cube {!r}.'.format(cube.name()))
@@ -415,8 +416,7 @@ def area_weights(cube, normalize=False):
 
     # Create 2D weights from bounds.
     # Use the geographical area as the weight for each cell
-    # Convert latitudes to co-latitude. I.e from -90 --> +90  to  0 --> pi
-    ll_weights = _quadrant_area(lat.bounds + np.pi / 2.,
+    ll_weights = _quadrant_area(lat.bounds,
                                 lon.bounds, radius_of_earth)
 
     # Normalize the weights if necessary.
@@ -575,7 +575,7 @@ def project(cube, target_proj, nx=None, ny=None):
 
     """
     try:
-        lat_coord, lon_coord = _get_lat_lon_coords(cube)
+        lon_coord, lat_coord = _get_lon_lat_coords(cube)
     except IndexError:
         raise ValueError('Cannot get latitude/longitude '
                          'coordinates from cube {!r}.'.format(cube.name()))
@@ -609,7 +609,7 @@ def project(cube, target_proj, nx=None, ny=None):
     source_x = lon_coord.points
     source_y = lat_coord.points
     if source_x.ndim != 2 or source_y.ndim != 2:
-        source_x, source_y = np.meshgrid(source_x, source_y)
+        source_x, source_y = _meshgrid(source_x, source_y)
 
     # Calculate target grid
     target_cs = None
@@ -1062,7 +1062,7 @@ def rotate_winds(u_cube, v_cube, target_cs):
 
     # Convert points to 2D, if not already, and determine dims.
     if x.ndim == y.ndim == 1:
-        x, y = np.meshgrid(x, y)
+        x, y = _meshgrid(x, y)
         dims = (y_dims[0], x_dims[0])
     else:
         dims = x_dims
