@@ -13,8 +13,8 @@ except NameError:
 
 def get_target_pkg():
     return target_pkg.target_pkg(__name__, __file__, pyke_version, loader, {
-         ('iris.fileformats._pyke_rules', '', 'fc_rules_cf.krb'):
-           [1485951831.612, 'fc_rules_cf_fc.py'],
+         ('', '', 'fc_rules_cf.krb'):
+           [1519243679.0532496, 'fc_rules_cf_fc.py'],
         },
         compiler_version)
 
