@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2016, Met Office
+# (C) British Crown Copyright 2010 - 2018, Met Office
 #
 # This file is part of Iris.
 #
@@ -37,7 +37,7 @@ import re
 import string
 import warnings
 
-import biggus
+import dask.array as da
 import netCDF4
 import numpy as np
 import numpy.ma as ma
@@ -48,6 +48,7 @@ import iris.analysis
 from iris.aux_factory import HybridHeightFactory, HybridPressureFactory, \
     OceanSigmaZFactory, OceanSigmaFactory, OceanSFactory, OceanSg1Factory, \
     OceanSg2Factory
+import iris.config
 import iris.coord_systems
 import iris.coords
 import iris.cube
@@ -56,7 +57,7 @@ import iris.fileformats.cf
 import iris.fileformats._pyke_rules
 import iris.io
 import iris.util
-
+from iris._lazy_data import as_lazy_data
 
 # Show Pyke inference engine statistics.
 DEBUG = False
@@ -90,7 +91,7 @@ _CF_ATTRS = ['add_offset', 'ancillary_variables', 'axis', 'bounds', 'calendar',
 
 # CF attributes that should not be global.
 _CF_DATA_ATTRS = ['flag_masks', 'flag_meanings', 'flag_values',
-                  'instance_dimension', 'sample_dimension',
+                  'instance_dimension', 'missing_value', 'sample_dimension',
                   'standard_error_multiplier']
 
 # CF attributes that should only be global.
@@ -391,10 +392,10 @@ class NetCDFDataProxy(object):
         try:
             variable = dataset.variables[self.variable_name]
             # Get the NetCDF variable data and slice.
-            data = variable[keys]
+            var = variable[keys]
         finally:
             dataset.close()
-        return data
+        return np.asanyarray(var)
 
     def __repr__(self):
         fmt = '<{self.__class__.__name__} shape={self.shape}' \
@@ -504,7 +505,7 @@ def _load_cube(engine, cf, cf_var, filename):
                          netCDF4.default_fillvals[cf_var.dtype.str[1:]])
     proxy = NetCDFDataProxy(cf_var.shape, dummy_data.dtype,
                             filename, cf_var.cf_name, fill_value)
-    data = biggus.OrthoArrayAdapter(proxy)
+    data = as_lazy_data(proxy)
     cube = iris.cube.Cube(data)
 
     # Reset the pyke inference engine.
@@ -575,12 +576,13 @@ def _load_aux_factory(engine, cube):
                         'ocean_s_coordinate_g2']:
         def coord_from_term(term):
             # Convert term names to coordinates (via netCDF variable names).
-            name = engine.requires['formula_terms'][term]
-            for coord, cf_var_name in engine.provides['coordinates']:
-                if cf_var_name == name:
-                    return coord
-            warnings.warn('Unable to find coordinate for variable '
-                          '{!r}'.format(name))
+            name = engine.requires['formula_terms'].get(term, None)
+            if name is not None:
+                for coord, cf_var_name in engine.provides['coordinates']:
+                    if cf_var_name == name:
+                        return coord
+                warnings.warn('Unable to find coordinate for variable '
+                              '{!r}'.format(name))
 
         if formula_type == 'atmosphere_hybrid_height_coordinate':
             delta = coord_from_term('a')
@@ -591,28 +593,31 @@ def _load_aux_factory(engine, cube):
             # Hybrid pressure has two valid versions of its formula terms:
             # "p0: var1 a: var2 b: var3 ps: var4" or
             # "ap: var1 b: var2 ps: var3" where "ap = p0 * a"
-            try:
-                # Attempt to get the "ap" term.
-                delta = coord_from_term('ap')
-            except (KeyError, ValueError):
+            # Attempt to get the "ap" term.
+            delta = coord_from_term('ap')
+            if delta is None:
                 # The "ap" term is unavailable, so try getting terms "p0"
                 # and "a" terms in order to derive an "ap" equivalent term.
                 coord_p0 = coord_from_term('p0')
-                if coord_p0.shape != (1,):
-                    msg = 'Expecting {!r} to be a scalar reference pressure ' \
-                        'coordinate, got shape {!r}'.format(coord_p0.var_name,
-                                                            coord_p0.shape)
-                    raise ValueError(msg)
-                if coord_p0.has_bounds():
-                    msg = 'Ignoring atmosphere hybrid sigma pressure scalar ' \
-                        'coordinate {!r} bounds.'.format(coord_p0.name())
-                    warnings.warn(msg)
-                coord_a = coord_from_term('a')
-                delta = coord_a * coord_p0.points[0]
-                delta.units = coord_a.units * coord_p0.units
-                delta.rename('vertical pressure')
-                delta.var_name = 'ap'
-                cube.add_aux_coord(delta, cube.coord_dims(coord_a))
+                if coord_p0 is not None:
+                    if coord_p0.shape != (1,):
+                        msg = 'Expecting {!r} to be a scalar reference ' \
+                            'pressure coordinate, got shape {!r}'.format(
+                                coord_p0.var_name,
+                                coord_p0.shape)
+                        raise ValueError(msg)
+                    if coord_p0.has_bounds():
+                        msg = 'Ignoring atmosphere hybrid sigma pressure ' \
+                            'scalar coordinate {!r} bounds.'.format(
+                                coord_p0.name())
+                        warnings.warn(msg)
+                    coord_a = coord_from_term('a')
+                    if coord_a is not None:
+                        delta = coord_a * coord_p0.points[0]
+                        delta.units = coord_a.units * coord_p0.units
+                        delta.rename('vertical pressure')
+                        delta.var_name = 'ap'
+                        cube.add_aux_coord(delta, cube.coord_dims(coord_a))
 
             sigma = coord_from_term('b')
             surface_air_pressure = coord_from_term('ps')
@@ -737,6 +742,26 @@ def _setncattr(variable, name, attribute):
     return variable.setncattr(name, attribute)
 
 
+class _FillValueMaskCheckAndStoreTarget(object):
+    """
+    To be used with da.store. Remembers whether any element was equal to a
+    given value and whether it was masked, before passing the chunk to the
+    given target.
+
+    """
+    def __init__(self, target, fill_value=None):
+        self.target = target
+        self.fill_value = fill_value
+        self.contains_value = False
+        self.is_masked = False
+
+    def __setitem__(self, keys, arr):
+        if self.fill_value is not None:
+            self.contains_value = self.contains_value or self.fill_value in arr
+        self.is_masked = self.is_masked or ma.is_masked(arr)
+        self.target[keys] = arr
+
+
 class Saver(object):
     """A manager for saving netcdf files."""
 
@@ -808,7 +833,7 @@ class Saver(object):
     def write(self, cube, local_keys=None, unlimited_dimensions=None,
               zlib=False, complevel=4, shuffle=True, fletcher32=False,
               contiguous=False, chunksizes=None, endian='native',
-              least_significant_digit=None, packing=None):
+              least_significant_digit=None, packing=None, fill_value=None):
         """
         Wrapper for saving cubes to a NetCDF file.
 
@@ -826,13 +851,11 @@ class Saver(object):
 
         * unlimited_dimensions (iterable of strings and/or
           :class:`iris.coords.Coord` objects):
-            Explicit list of coordinate names (or coordinate objects)
+            List of coordinate names (or coordinate objects)
             corresponding to coordinate dimensions of `cube` to save with the
-            NetCDF dimension variable length 'UNLIMITED'. By default, the
-            outermost (first) dimension for each cube is used. Only the
-            'NETCDF4' format supports multiple 'UNLIMITED' dimensions. To save
-            no unlimited dimensions, use `unlimited_dimensions=[]` (an empty
-            list).
+            NetCDF dimension variable length 'UNLIMITED'. By default, no
+            unlimited dimensions are saved. Only the 'NETCDF4' format
+            supports multiple 'UNLIMITED' dimensions.
 
         * zlib (bool):
             If `True`, the data will be compressed in the netCDF file using
@@ -895,14 +918,18 @@ class Saver(object):
             http://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html#bp_Packed-Data-Values
             If this argument is a type (or type string), appropriate values of
             scale_factor and add_offset will be automatically calculated based
-            on `cube.data` and possible masking. For masked data, `fill_value`
-            is taken from netCDF4.default_fillvals. For more control, pass a
-            dict with one or more of the following keys: `dtype` (required),
-            `scale_factor`, `add_offset`, and `fill_value`. Note that automatic
-            calculation of packing parameters will trigger loading of lazy
-            data; set them manually using a dict to avoid this. The default is
-            `None`, in which case the datatype is determined from the cube and
-            no packing will occur.
+            on `cube.data` and possible masking. For more control, pass a dict
+            with one or more of the following keys: `dtype` (required),
+            `scale_factor` and `add_offset`. Note that automatic calculation of
+            packing parameters will trigger loading of lazy data; set them
+            manually using a dict to avoid this. The default is `None`, in
+            which case the datatype is determined from the cube and no packing
+            will occur.
+
+        * fill_value:
+            The value to use for the `_FillValue` attribute on the netCDF
+            variable. If `packing` is specified the value of `fill_value`
+            should be in the domain of the packed data.
 
         Returns:
             None.
@@ -913,19 +940,9 @@ class Saver(object):
             `chunksizes` and `endian` keywords are silently ignored for netCDF
             3 files that do not use HDF5.
 
-        .. deprecated:: 1.8.0
-
-            NetCDF default saving behaviour currently assigns the outermost
-            dimension as unlimited. This behaviour is to be deprecated, in
-            favour of no automatic assignment. To switch to the new behaviour,
-            set `iris.FUTURE.netcdf_no_unlimited` to True.
-
-        """
+            """
         if unlimited_dimensions is None:
-            if iris.FUTURE.netcdf_no_unlimited:
                 unlimited_dimensions = []
-            else:
-                _no_unlim_dep_warning()
 
         cf_profile_available = (iris.site_configuration.get('cf_profile') not
                                 in [None, False])
@@ -951,7 +968,8 @@ class Saver(object):
             cube, dimension_names, local_keys, zlib=zlib, complevel=complevel,
             shuffle=shuffle, fletcher32=fletcher32, contiguous=contiguous,
             chunksizes=chunksizes, endian=endian,
-            least_significant_digit=least_significant_digit, packing=packing)
+            least_significant_digit=least_significant_digit, packing=packing,
+            fill_value=fill_value)
 
         # Add coordinate variables.
         self._add_dim_coords(cube, dimension_names)
@@ -1058,29 +1076,23 @@ class Saver(object):
 
         * unlimited_dimensions (iterable of strings and/or
           :class:`iris.coords.Coord` objects):
-            List of coordinates to make unlimited. By default, the
-            outermost dimension is made unlimited.
+            List of coordinates to make unlimited (None by default).
 
         Returns:
             None.
 
         """
         unlimited_dim_names = []
-        if (unlimited_dimensions is None and
-                not iris.FUTURE.netcdf_no_unlimited):
-            if dimension_names:
-                unlimited_dim_names.append(dimension_names[0])
-        else:
-            for coord in unlimited_dimensions:
-                try:
-                    coord = cube.coord(name_or_coord=coord, dim_coords=True)
-                except iris.exceptions.CoordinateNotFoundError:
-                    # coordinate isn't used for this cube, but it might be
-                    # used for a different one
-                    pass
-                else:
-                    dim_name = self._get_coord_variable_name(cube, coord)
-                    unlimited_dim_names.append(dim_name)
+        for coord in unlimited_dimensions:
+            try:
+                coord = cube.coord(name_or_coord=coord, dim_coords=True)
+            except iris.exceptions.CoordinateNotFoundError:
+                # coordinate isn't used for this cube, but it might be
+                # used for a different one
+                pass
+            else:
+                dim_name = self._get_coord_variable_name(cube, coord)
+                unlimited_dim_names.append(dim_name)
 
         for dim_name in dimension_names:
             if dim_name not in self._dataset.dimensions:
@@ -1353,12 +1365,13 @@ class Saver(object):
         return coord.standard_name, coord.long_name, units
 
     def _ensure_valid_dtype(self, values, src_name, src_object):
-        # NetCDF3 does not support int64 or unsigned ints, so we check
-        # if we can store them as int32 instead.
+        # NetCDF3 and NetCDF4 classic do not support int64 or unsigned ints,
+        # so we check if we can store them as int32 instead.
         if ((np.issubdtype(values.dtype, np.int64) or
                 np.issubdtype(values.dtype, np.unsignedinteger)) and
                 self._dataset.file_format in ('NETCDF3_CLASSIC',
-                                              'NETCDF3_64BIT')):
+                                              'NETCDF3_64BIT',
+                                              'NETCDF4_CLASSIC')):
             # Cast to an integer type supported by netCDF3.
             if not np.can_cast(values.max(), np.int32) or \
                     not np.can_cast(values.min(), np.int32):
@@ -1491,9 +1504,18 @@ class Saver(object):
         cf_dimensions = [dimension_names[dim] for dim in
                          cube.cell_measure_dims(cell_measure)]
 
+        # Get the data values.
+        data = cell_measure.data
+
+        # Disallow saving of *masked* cell measures.
+        if ma.is_masked(data):
+            # We can't save masked points properly, as we don't maintain a
+            # suitable fill_value.  (Load will not record one, either).
+            msg = "Cell measures with missing data are not supported."
+            raise ValueError(msg)
+
         # Get the values in a form which is valid for the file format.
-        data = self._ensure_valid_dtype(cell_measure.data, 'coordinate',
-                                        cell_measure)
+        data = self._ensure_valid_dtype(data, 'coordinate', cell_measure)
 
         # Create the CF-netCDF variable.
         cf_var = self._dataset.createVariable(
@@ -1827,7 +1849,7 @@ class Saver(object):
             _setncattr(cf_var_cube, 'grid_mapping', cs.grid_mapping_name)
 
     def _create_cf_data_variable(self, cube, dimension_names, local_keys=None,
-                                 **kwargs):
+                                 packing=None, fill_value=None, **kwargs):
         """
         Create CF-netCDF data variable for the cube and any associated grid
         mapping.
@@ -1842,8 +1864,11 @@ class Saver(object):
         Kwargs:
 
         * local_keys (iterable of strings):
-            An interable of cube attribute keys. Any cube attributes
-            with matching keys will become attributes on the data variable.
+            * see :func:`iris.fileformats.netcdf.Saver.write`
+        * packing (type or string or dict or list):
+            * see :func:`iris.fileformats.netcdf.Saver.write`
+        * fill_value:
+            * see :func:`iris.fileformats.netcdf.Saver.write`
 
         All other keywords are passed through to the dataset's `createVariable`
         method.
@@ -1852,47 +1877,41 @@ class Saver(object):
             The newly created CF-netCDF data variable.
 
         """
-        if 'packing' in kwargs:
-            packing = kwargs.pop('packing')
-            if packing:
-                scale_factor = None
-                add_offset = None
-                fill_value = None
-                if isinstance(packing, dict):
-                    if 'dtype' not in packing:
-                        msg = "The dtype attribute is required for packing."
-                        raise ValueError(msg)
-                    dtype = np.dtype(packing['dtype'])
-                    if 'scale_factor' in packing:
-                        scale_factor = packing['scale_factor']
-                    if 'add_offset' in packing:
-                        add_offset = packing['add_offset']
-                    if 'fill_value' in packing:
-                        fill_value = packing['fill_value']
+
+        if packing:
+            if isinstance(packing, dict):
+                if 'dtype' not in packing:
+                    msg = "The dtype attribute is required for packing."
+                    raise ValueError(msg)
+                dtype = np.dtype(packing['dtype'])
+                scale_factor = packing.get('scale_factor', None)
+                add_offset = packing.get('add_offset', None)
+                valid_keys = {'dtype', 'scale_factor', 'add_offset'}
+                invalid_keys = set(packing.keys()) - valid_keys
+                if invalid_keys:
+                    msg = ("Invalid packing key(s) found: '{}'. The valid "
+                           "keys are '{}'.".format("', '".join(invalid_keys),
+                                                   "', '".join(valid_keys)))
+                    raise ValueError(msg)
+            else:
+                # We compute the scale_factor and add_offset based on the
+                # min/max of the data. This requires the data to be loaded.
+                masked = ma.isMaskedArray(cube.data)
+                dtype = np.dtype(packing)
+                cmax = cube.data.max()
+                cmin = cube.data.min()
+                n = dtype.itemsize * 8
+                if masked:
+                    scale_factor = (cmax - cmin)/(2**n-2)
                 else:
-                    masked = ma.isMaskedArray(cube.data)
-                    dtype = np.dtype(packing)
-                    cmax = cube.data.max()
-                    cmin = cube.data.min()
-                    n = dtype.itemsize * 8
+                    scale_factor = (cmax - cmin)/(2**n-1)
+                if dtype.kind == 'u':
+                    add_offset = cmin
+                elif dtype.kind == 'i':
                     if masked:
-                        scale_factor = (cmax - cmin)/(2**n-2)
+                        add_offset = (cmax + cmin)/2
                     else:
-                        scale_factor = (cmax - cmin)/(2**n-1)
-                    if dtype.kind == 'u':
-                        add_offset = cmin
-                    elif dtype.kind == 'i':
-                        if masked:
-                            add_offset = (cmax + cmin)/2
-                        else:
-                            add_offset = cmin + 2**(n-1)*scale_factor
-                needs_fill_value = (cube.has_lazy_data() or
-                                    ma.isMaskedArray(cube.data))
-                if needs_fill_value and fill_value is None:
-                    dtstr = dtype.str[1:]
-                    fill_value = netCDF4.default_fillvals[dtstr]
-        else:
-            packing = None
+                        add_offset = cmin + 2**(n-1)*scale_factor
 
         def set_packing_ncattrs(cfvar):
             """Set netCDF packing attributes."""
@@ -1911,41 +1930,61 @@ class Saver(object):
             self._dataset.file_format in ('NETCDF3_CLASSIC',
                                           'NETCDF3_64BIT')):
 
-            if packing is None:
-                # Determine whether there is a cube MDI value.
-                if ma.isMaskedArray(cube.data):
-                    fill_value = cube.data.fill_value
-                else:
-                    fill_value = None
-
             # Get the values in a form which is valid for the file format.
             data = self._ensure_valid_dtype(cube.data, 'cube', cube)
 
-            if packing is None:
-                dtype = data.dtype.newbyteorder('=')
-
-            # Create the cube CF-netCDF data variable with data payload.
-            cf_var = self._dataset.createVariable(
-                cf_name, dtype, dimension_names,
-                fill_value=fill_value, **kwargs)
-            set_packing_ncattrs(cf_var)
-            cf_var[:] = data
-
+            def store(data, cf_var, fill_value):
+                cf_var[:] = data
+                is_masked = ma.is_masked(data)
+                contains_value = fill_value is not None and fill_value in data
+                return is_masked, contains_value
         else:
-            # Create the cube CF-netCDF data variable.
-            # Explicitly assign the fill_value, which will be the type default
-            # in the case of an unmasked array.
-            if packing is None:
-                fill_value = cube.lazy_data().fill_value
-                dtype = cube.lazy_data().dtype.newbyteorder('=')
+            data = cube.lazy_data()
 
-            cf_var = self._dataset.createVariable(
-                cf_name, dtype,
-                dimension_names, fill_value=fill_value,
-                **kwargs)
-            set_packing_ncattrs(cf_var)
-            # stream the data
-            biggus.save([cube.lazy_data()], [cf_var], masked=True)
+            def store(data, cf_var, fill_value):
+                # Store lazy data and check whether it is masked and contains
+                # the fill value
+                target = _FillValueMaskCheckAndStoreTarget(cf_var, fill_value)
+                da.store([data], [target])
+                return target.is_masked, target.contains_value
+
+        if not packing:
+            dtype = data.dtype.newbyteorder('=')
+
+        # Create the cube CF-netCDF data variable with data payload.
+        cf_var = self._dataset.createVariable(cf_name, dtype, dimension_names,
+                                              fill_value=fill_value,
+                                              **kwargs)
+        set_packing_ncattrs(cf_var)
+
+        # If packing attributes are specified, don't bother checking whether
+        # the fill value is in the data.
+        if packing:
+            fill_value_to_check = None
+        elif fill_value is not None:
+            fill_value_to_check = fill_value
+        else:
+            fill_value_to_check = netCDF4.default_fillvals[dtype.str[1:]]
+
+        # Store the data and check if it is masked and contains the fill value
+        is_masked, contains_fill_value = store(data, cf_var,
+                                               fill_value_to_check)
+
+        if dtype.itemsize == 1 and fill_value is None:
+            if is_masked:
+                msg = ("Cube '{}' contains byte data with masked points, but "
+                       "no fill_value keyword was given. As saved, these "
+                       "points will read back as valid values. To save as "
+                       "masked byte data, please explicitly specify the "
+                       "'fill_value' keyword.")
+                warnings.warn(msg.format(cube.name()))
+        elif contains_fill_value:
+            msg = ("Cube '{}' contains unmasked data points equal to the "
+                   "fill-value, {}. As saved, these points will read back "
+                   "as missing data. To save these as normal values, please "
+                   "specify a 'fill_value' keyword not equal to any valid "
+                   "data points.")
+            warnings.warn(msg.format(cube.name(), fill_value))
 
         if cube.standard_name:
             _setncattr(cf_var, 'standard_name', cube.standard_name)
@@ -2032,7 +2071,7 @@ class Saver(object):
 def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
          unlimited_dimensions=None, zlib=False, complevel=4, shuffle=True,
          fletcher32=False, contiguous=False, chunksizes=None, endian='native',
-         least_significant_digit=None, packing=None):
+         least_significant_digit=None, packing=None, fill_value=None):
     """
     Save cube(s) to a netCDF file, given the cube and the filename.
 
@@ -2043,7 +2082,7 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
     * Keyword arguments specifying how to save the data are applied
       to each cube. To use different settings for different cubes, use
       the NetCDF Context manager (:class:`~Saver`) directly.
-    * The save process will stream the data payload to the file using biggus,
+    * The save process will stream the data payload to the file using dask,
       enabling large data payloads to be saved and maintaining the 'lazy'
       status of the cube's data payload, unless the netcdf_format is explicitly
       specified to be 'NETCDF3' or 'NETCDF3_CLASSIC'.
@@ -2070,12 +2109,11 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
 
     * unlimited_dimensions (iterable of strings and/or
       :class:`iris.coords.Coord` objects):
-        Explicit list of coordinate names (or coordinate objects) corresponding
+        List of coordinate names (or coordinate objects) corresponding
         to coordinate dimensions of `cube` to save with the NetCDF dimension
-        variable length 'UNLIMITED'. By default, the outermost (first)
-        dimension for each cube is used. Only the 'NETCDF4' format supports
-        multiple 'UNLIMITED' dimensions. To save no unlimited dimensions, use
-        `unlimited_dimensions=[]` (an empty list).
+        variable length 'UNLIMITED'. By default, no unlimited dimensions are
+        saved. Only the 'NETCDF4' format supports multiple 'UNLIMITED'
+        dimensions.
 
     * zlib (bool):
         If `True`, the data will be compressed in the netCDF file using gzip
@@ -2136,16 +2174,24 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
         This provides support for netCDF data packing as described in
         http://www.unidata.ucar.edu/software/netcdf/docs/BestPractices.html#bp_Packed-Data-Values
         If this argument is a type (or type string), appropriate values of
-        scale_factor and add_offset will be automatically calculated based on
-        `cube.data` and possible masking. For masked data, `fill_value` is
-        taken from netCDF4.default_fillvals. For more control, pass a dict with
-        one or more of the following keys: `dtype` (required), `scale_factor`,
-        `add_offset`, and `fill_value`. To save multiple cubes with different
-        packing parameters, pass an iterable of types, strings, dicts, or
-        `None`, one for each cube.  Note that automatic calculation of packing
-        parameters will trigger loading of lazy data; set them manually using a
-        dict to avoid this. The default is `None`, in which case the datatype
-        is determined from the cube and no packing will occur.
+        scale_factor and add_offset will be automatically calculated based
+        on `cube.data` and possible masking. For more control, pass a dict with
+        one or more of the following keys: `dtype` (required), `scale_factor`
+        and `add_offset`. Note that automatic calculation of packing parameters
+        will trigger loading of lazy data; set them manually using a dict to
+        avoid this. The default is `None`, in which case the datatype is
+        determined from the cube and no packing will occur. If this argument is
+        a list it must have the same number of elements as `cube` if `cube` is
+        a `:class:`iris.cube.CubeList`, or one element, and each element of
+        this argument will be applied to each cube separately.
+
+    * fill_value (numeric or list):
+        The value to use for the `_FillValue` attribute on the netCDF variable.
+        If `packing` is specified the value of `fill_value` should be in the
+        domain of the packed data. If this argument is a list it must have the
+        same number of elements as `cube` if `cube` is a
+        `:class:`iris.cube.CubeList`, or a single element, and each element of
+        this argument will be applied to each cube separately.
 
     Returns:
         None.
@@ -2160,19 +2206,9 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
 
         NetCDF Context manager (:class:`~Saver`).
 
-    .. deprecated:: 1.8.0
-
-        NetCDF default saving behaviour currently assigns the outermost
-        dimensions to unlimited. This behaviour is to be deprecated, in
-        favour of no automatic assignment. To switch to the new behaviour,
-        set `iris.FUTURE.netcdf_no_unlimited` to True.
-
     """
     if unlimited_dimensions is None:
-        if iris.FUTURE.netcdf_no_unlimited:
             unlimited_dimensions = []
-        else:
-            _no_unlim_dep_warning()
 
     if isinstance(cube, iris.cube.Cube):
         cubes = iris.cube.CubeList()
@@ -2236,15 +2272,36 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
                 raise ValueError(msg)
         packspecs = packing
 
+    # Make fill-value(s) into an iterable over cubes.
+    if isinstance(fill_value, six.string_types):
+        # Strings are awkward -- handle separately.
+        fill_values = repeat(fill_value)
+    else:
+        try:
+            fill_values = tuple(fill_value)
+        except TypeError:
+            fill_values = repeat(fill_value)
+        else:
+            if len(fill_values) != len(cubes):
+                msg = ('If fill_value is a list, it must have the '
+                       'same number of elements as the cube argument.')
+                raise ValueError(msg)
+
     # Initialise Manager for saving
     with Saver(filename, netcdf_format) as sman:
         # Iterate through the cubelist.
-        for cube, packspec in zip(cubes, packspecs):
+        for cube, packspec, fill_value in zip(cubes, packspecs, fill_values):
             sman.write(cube, local_keys, unlimited_dimensions, zlib, complevel,
                        shuffle, fletcher32, contiguous, chunksizes, endian,
-                       least_significant_digit, packing=packspec)
+                       least_significant_digit, packing=packspec,
+                       fill_value=fill_value)
 
-        conventions = CF_CONVENTIONS_VERSION
+        if iris.config.netcdf.conventions_override:
+            # Set to the default if custom conventions are not available.
+            conventions = cube.attributes.get('Conventions',
+                                              CF_CONVENTIONS_VERSION)
+        else:
+            conventions = CF_CONVENTIONS_VERSION
 
         # Perform a CF patch of the conventions attribute.
         cf_profile_available = (iris.site_configuration.get('cf_profile') not
@@ -2261,12 +2318,3 @@ def save(cube, filename, netcdf_format='NETCDF4', local_keys=None,
 
         # Add conventions attribute.
         sman.update_global_attributes(Conventions=conventions)
-
-
-def _no_unlim_dep_warning():
-    msg = ('NetCDF default saving behaviour currently assigns the '
-           'outermost dimensions to unlimited. This behaviour is to be '
-           'deprecated, in favour of no automatic assignment. To switch '
-           'to the new behaviour, set iris.FUTURE.netcdf_no_unlimited to '
-           'True.')
-    warn_deprecated(msg)
