@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2016, Met Office
+# (C) British Crown Copyright 2010 - 2017, Met Office
 #
 # This file is part of Iris.
 #
@@ -30,6 +30,7 @@ import datetime
 
 import cartopy.crs as ccrs
 import cartopy.mpl.geoaxes
+from cartopy.geodesic import Geodesic
 import matplotlib.axes
 import matplotlib.collections as mpl_collections
 import matplotlib.dates as mpl_dates
@@ -44,9 +45,11 @@ import numpy.ma as ma
 import iris.cube
 import iris.analysis.cartography as cartography
 import iris.coords
+import iris.coord_systems
 from iris.exceptions import IrisError
 # Importing iris.palette to register the brewer palettes.
 import iris.palette
+from iris.util import _meshgrid
 
 
 # Cynthia Brewer citation text.
@@ -183,11 +186,15 @@ def _get_plot_defn(cube, mode, ndims=2):
                     coords[axes.index(axis)] = coord
 
     # Re-order the coordinates to achieve the preferred
-    # horizontal/vertical associations.
+    # horizontal/vertical associations. If we can't associate
+    # an axis to order the coordinates, fall back to using the cube dimension
+    # followed by the name of the coordinate.
     def sort_key(coord):
         order = {'X': 2, 'T': 1, 'Y': -1, 'Z': -2}
         axis = guess_axis(coord)
-        return (order.get(axis, 0), coord and coord.name())
+        return (order.get(axis, 0),
+                coords.index(coord),
+                coord and coord.name())
     sorted_coords = sorted(coords, key=sort_key)
 
     transpose = (sorted_coords != coords)
@@ -481,6 +488,79 @@ def _get_plot_objects(args):
     return u_object, v_object, u, v, args
 
 
+def _get_geodesic_params(globe):
+    # Derive the semimajor axis and flattening values for a given globe from
+    # its attributes. If the values are under specified, raise a ValueError
+    flattening = globe.flattening
+    semimajor = globe.semimajor_axis
+    try:
+        if semimajor is None:
+            # Has semiminor or raises error
+            if flattening is None:
+                # Has inverse flattening or raises error
+                flattening = 1. / globe.inverse_flattening
+            semimajor = globe.semiminor_axis / (1. - flattening)
+        elif flattening is None:
+            if globe.semiminor_axis is not None:
+                flattening = ((semimajor - globe.semiminor_axis) /
+                              float(semimajor))
+            else:
+                # Has inverse flattening or raises error
+                flattening = 1. / globe.inverse_flattening
+    except TypeError:
+        # One of the required attributes was None
+        raise ValueError('The globe was underspecified.')
+
+    return semimajor, flattening
+
+
+def _shift_plot_sections(u_object, u, v):
+    """
+    Shifts subsections of u by multiples of 360 degrees within ranges
+    defined by the points where the line should cross over the 0/360 degree
+    longitude boundary.
+
+    e.g. [ 300, 100, 200, 300, 100, 300 ] => [ 300, 460, 560, 660, 820, 660 ]
+
+    """
+    # Convert coordinates to true lat-lon
+    src_crs = u_object.coord_system.as_cartopy_crs() \
+        if u_object.coord_system is not None else ccrs.Geodetic()
+    tgt_crs = ccrs.Geodetic(globe=src_crs.globe)
+    tgt_proj = ccrs.PlateCarree(globe=src_crs.globe)
+
+    points = tgt_crs.transform_points(src_crs, u, v)
+    startpoints = points[:-1, :2]
+    endpoints = points[1:, :2]
+    proj_x, proj_y, _ = tgt_proj.transform_points(src_crs, u, v).T
+
+    # Calculate the inverse geodesic for each pair of points in turn, and
+    # convert the start point's azimuth into a vector in the source coordinate
+    # system.
+    try:
+        radius, flattening = _get_geodesic_params(src_crs.globe)
+        geodesic = Geodesic(radius, flattening)
+    except ValueError:
+        geodesic = Geodesic()
+    dists, azms, _ = geodesic.inverse(startpoints, endpoints).T
+    azms_lon = np.sin(np.deg2rad(azms))
+    azms_lat = np.cos(np.deg2rad(azms))
+    azms_u, _ = src_crs.transform_vectors(tgt_proj, proj_x[:-1], proj_y[:-1],
+                                          azms_lon, azms_lat)
+
+    # Use the grid longitude values and the geodesic azimuth to determine
+    # the points where the line should cross the 0/360 degree boundary, and
+    # in which direction
+    lwraps = np.logical_and(u[1:] > u[:-1], azms_u < 0)
+    rwraps = np.logical_and(u[1:] < u[:-1], azms_u > 0)
+    shifts = np.where(rwraps, 1, 0) - np.where(lwraps, 1, 0)
+    shift_vals = shifts.cumsum() * u_object.units.modulus
+    new_u = np.empty_like(u)
+    new_u[0] = u[0]
+    new_u[1:] = u[1:] + shift_vals
+    return new_u
+
+
 def _draw_1d_from_points(draw_method_name, arg_func, *args, **kwargs):
     # NB. In the interests of clarity we use "u" to refer to the horizontal
     # axes on the matplotlib plot and "v" for the vertical axes.
@@ -518,6 +598,10 @@ def _draw_1d_from_points(draw_method_name, arg_func, *args, **kwargs):
         # the transform keyword.
         kwargs = _ensure_cartopy_axes_and_determine_kwargs(u_object, v_object,
                                                            kwargs)
+        if draw_method_name == 'plot' and \
+                u_object.standard_name not in ('projection_x_coordinate',
+                                               'projection_y_coordinate'):
+            u = _shift_plot_sections(u_object, u, v)
 
     axes = kwargs.pop('axes', None)
     draw_method = getattr(axes if axes else plt, draw_method_name)
@@ -584,7 +668,18 @@ def _ensure_cartopy_axes_and_determine_kwargs(x_coord, y_coord, kwargs):
     # Ensure the current axes are a cartopy.mpl.GeoAxes instance.
     axes = kwargs.get('axes')
     if axes is None:
-        _replace_axes_with_cartopy_axes(cartopy_proj)
+        if (isinstance(cs, iris.coord_systems.RotatedGeogCS) and
+           x_coord.points.max() > 180 and x_coord.points.max() < 360 and
+           x_coord.points.min() > 0):
+            # The RotatedGeogCS has 0 - 360 extent, different from the
+            # assumptions made by Cartopy: rebase longitudes for the map axes
+            # to set the datum longitude to the International Date Line.
+            cs_kwargs = cs._ccrs_kwargs()
+            cs_kwargs['central_rotated_longitude'] = 180.0
+            adapted_cartopy_proj = ccrs.RotatedPole(**cs_kwargs)
+            _replace_axes_with_cartopy_axes(adapted_cartopy_proj)
+        else:
+            _replace_axes_with_cartopy_axes(cartopy_proj)
     elif axes and not isinstance(axes, cartopy.mpl.geoaxes.GeoAxes):
         raise TypeError("The supplied axes instance must be a cartopy "
                         "GeoAxes instance.")
@@ -613,7 +708,7 @@ def _map_common(draw_method_name, arg_func, mode, cube, plot_defn,
     y_coord, x_coord = plot_defn.coords
     if mode == iris.coords.POINT_MODE:
         if x_coord.ndim == y_coord.ndim == 1:
-            x, y = np.meshgrid(x_coord.points, y_coord.points)
+            x, y = _meshgrid(x_coord.points, y_coord.points)
         elif x_coord.ndim == y_coord.ndim == 2:
             x = x_coord.points
             y = y_coord.points
@@ -621,8 +716,8 @@ def _map_common(draw_method_name, arg_func, mode, cube, plot_defn,
             raise ValueError("Expected 1D or 2D XY coords")
     else:
         try:
-            x, y = np.meshgrid(x_coord.contiguous_bounds(),
-                               y_coord.contiguous_bounds())
+            x, y = _meshgrid(x_coord.contiguous_bounds(),
+                             y_coord.contiguous_bounds())
         # Exception translation.
         except iris.exceptions.CoordinateMultiDimError:
             raise ValueError("Could not get XY grid from bounds. "
