@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2013 - 2016, Met Office
+# (C) British Crown Copyright 2013 - 2017, Met Office
 #
 # This file is part of Iris.
 #
@@ -28,6 +28,7 @@ import unittest
 
 import cf_units
 import matplotlib.units
+import netCDF4
 import netcdftime
 import numpy as np
 
@@ -100,11 +101,18 @@ class TestAsSeries(tests.IrisTest):
         time_coord = DimCoord([0, 100.1, 200.2, 300.3, 400.4],
                               long_name="time", units=time_unit)
         cube.add_dim_coord(time_coord, 0)
-        expected_index = [netcdftime.datetime(2000, 1, 1, 0, 0),
-                          netcdftime.datetime(2000, 4, 11, 2, 24),
-                          netcdftime.datetime(2000, 7, 21, 4, 48),
-                          netcdftime.datetime(2000, 11, 1, 7, 12),
-                          netcdftime.datetime(2001, 2, 11, 9, 36)]
+        if netCDF4.__version__ > '1.2.4':
+            expected_index = [netcdftime.Datetime360Day(2000, 1, 1, 0, 0),
+                              netcdftime.Datetime360Day(2000, 4, 11, 2, 24),
+                              netcdftime.Datetime360Day(2000, 7, 21, 4, 48),
+                              netcdftime.Datetime360Day(2000, 11, 1, 7, 12),
+                              netcdftime.Datetime360Day(2001, 2, 11, 9, 36)]
+        else:
+            expected_index = [netcdftime.datetime(2000, 1, 1, 0, 0),
+                              netcdftime.datetime(2000, 4, 11, 2, 24),
+                              netcdftime.datetime(2000, 7, 21, 4, 48),
+                              netcdftime.datetime(2000, 11, 1, 7, 12),
+                              netcdftime.datetime(2001, 2, 11, 9, 36)]
         series = iris.pandas.as_series(cube)
         self.assertArrayEqual(series, cube.data)
         self.assertArrayEqual(series.index, expected_index)
@@ -236,8 +244,12 @@ class TestAsDataFrame(tests.IrisTest):
         time_coord = DimCoord([100.1, 200.2], long_name="time",
                               units=time_unit)
         cube.add_dim_coord(time_coord, 0)
-        expected_index = [netcdftime.datetime(2000, 4, 11, 2, 24),
-                          netcdftime.datetime(2000, 7, 21, 4, 48)]
+        if netCDF4.__version__ > '1.2.4':
+            expected_index = [netcdftime.Datetime360Day(2000, 4, 11, 2, 24),
+                              netcdftime.Datetime360Day(2000, 7, 21, 4, 48)]
+        else:
+            expected_index = [netcdftime.datetime(2000, 4, 11, 2, 24),
+                              netcdftime.datetime(2000, 7, 21, 4, 48)]
         expected_columns = [0, 1, 2, 3, 4]
         data_frame = iris.pandas.as_data_frame(cube)
         self.assertArrayEqual(data_frame, cube.data)
@@ -287,6 +299,13 @@ class TestAsDataFrame(tests.IrisTest):
         with self.assertRaises(ValueError):
             data_frame = iris.pandas.as_data_frame(cube, copy=False)
 
+    def test_copy_false_with_cube_view(self):
+        data = np.array([[0, 1, 2, 3, 4], [5, 6, 7, 8, 9]])
+        cube = Cube(data[:], long_name="foo")
+        data_frame = iris.pandas.as_data_frame(cube, copy=False)
+        data_frame[0][0] = 99
+        self.assertEqual(cube.data[0, 0], 99)
+
 
 @skip_pandas
 class TestSeriesAsCube(tests.IrisTest):
