@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2010 - 2016, Met Office
+# (C) British Crown Copyright 2010 - 2017, Met Office
 #
 # This file is part of Iris.
 #
@@ -348,7 +348,7 @@ class TestAttributePositive(tests.GraphicsTest):
     def test_2d_positive_up(self):
         path = tests.get_data_path(('NetCDF', 'testing',
                                     'small_theta_colpex.nc'))
-        cube = iris.load_cube(path)[0, :, 42, :]
+        cube = iris.load_cube(path, 'air_potential_temperature')[0, :, 42, :]
         qplt.pcolormesh(cube)
         self.check_graphic()
 
@@ -581,8 +581,10 @@ class CheckForWarningsMetaclass(type):
 
 
 @tests.skip_data
+@tests.iristest_timing_decorator
 class TestPcolorNoBounds(six.with_metaclass(CheckForWarningsMetaclass,
-                                            tests.GraphicsTest, SliceMixin)):
+                                            tests.GraphicsTest_nometa,
+                                            SliceMixin)):
     """
     Test the iris.plot.pcolor routine on a cube with coordinates
     that have no bounds.
@@ -595,8 +597,9 @@ class TestPcolorNoBounds(six.with_metaclass(CheckForWarningsMetaclass,
 
 
 @tests.skip_data
+@tests.iristest_timing_decorator
 class TestPcolormeshNoBounds(six.with_metaclass(CheckForWarningsMetaclass,
-                                                tests.GraphicsTest,
+                                                tests.GraphicsTest_nometa,
                                                 SliceMixin)):
     """
     Test the iris.plot.pcolormesh routine on a cube with coordinates
