@@ -1,4 +1,4 @@
-# (C) British Crown Copyright 2013 - 2016, Met Office
+# (C) British Crown Copyright 2013 - 2017, Met Office
 #
 # This file is part of Iris.
 #
@@ -22,12 +22,13 @@ from six.moves import (filter, input, map, range, zip)  # noqa
 # Import iris.tests first so that some things can be initialised before
 # importing anything else.
 import iris.tests as tests
+import iris.tests.stock as stock
 
 import copy
+from iris._lazy_data import as_lazy_data
 import numpy as np
 import unittest
 
-from biggus import NumpyArrayAdapter
 import iris
 from iris.util import new_axis
 
@@ -136,7 +137,7 @@ class Test(tests.IrisTest):
         self._assert_cube_notis(res, cube)
 
     def test_lazy_data(self):
-        cube = iris.cube.Cube(NumpyArrayAdapter(self.data))
+        cube = iris.cube.Cube(as_lazy_data(self.data))
         cube.add_aux_coord(iris.coords.DimCoord([1], standard_name='time'))
         res = new_axis(cube, 'time')
         self.assertTrue(cube.has_lazy_data())
@@ -144,7 +145,7 @@ class Test(tests.IrisTest):
         self.assertEqual(res.shape, (1,) + cube.shape)
 
     def test_masked_unit_array(self):
-        cube = tests.stock.simple_3d_mask()
+        cube = stock.simple_3d_mask()
         test_cube = cube[0, 0, 0]
         test_cube = new_axis(test_cube, 'longitude')
         test_cube = new_axis(test_cube, 'latitude')
